/*------------------------------------------------------------------------------
 bin/djv_view/color_sample.h
 
 Color Sample
 ------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_COLOR_SAMPLE_H
#define DJV_VIEW_COLOR_SAMPLE_H

#include <djv_ui/dialog.h>
#include "base.h"

namespace djv_view {
namespace view { class Widget; }
namespace color_sample
{

//------------------------------------------------------------------------------
// Options

enum SIZE { SIZE_1X1, SIZE_3X3, SIZE_5X5, SIZE_7X7, SIZE_9X9 };

//------------------------------------------------------------------------------
// Dialog

class Dialog : public ui::dialog::Base
{
public:
  
  Dialog();
  
  // Value.
  
  void set(const view::Widget *, const Vec2i & pick);
  void update(const view::Widget *, const Vec2i & pick);
  void del(const view::Widget *);
  
  // Widget.
  
  void show();
  void widget_del();
  
  // Layout.
  
  void update() { Base::update(); }

private:

  DJV_CALLBACK(Dialog, widget_callback, const image::Color &);
  DJV_CALLBACK(Dialog, size_callback, int);
  DJV_CALLBACK(Dialog, profile_callback, bool);
  DJV_CALLBACK(Dialog, lock_callback, bool);
  DJV_CALLBACK(Dialog, close_callback, bool);

  void widget_update();
  
  const view::Widget * _view;
  image::Color _value;
  Vec2i _pick;
  SIZE _size;
  bool _profile, _lock;
  bool _update;
  ui::color::Widget * _widget;
  ui::color::Swatch * _swatch;
  ui::choice::Widget * _size_widget;
  ui::button::Check * _profile_widget;
  ui::button::Check * _lock_widget;
};

//------------------------------------------------------------------------------
// Functions

int size(SIZE);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_size();

//------------------------------------------------------------------------------
// Global

Dialog * dialog();

void del();

}
}

#endif

