/*------------------------------------------------------------------------------
 bin/djv_view/histogram.h
 
 Image Histogram
 ---------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_HISTOGRAM_H
#define DJV_VIEW_HISTOGRAM_H

#include <djv_ui/group.h>
#include <djv_ui/dialog.h>
#include "view.h"

namespace djv_view {
namespace histogram
{

//------------------------------------------------------------------------------
// Widget

class Widget : public ui::group::Frame
{
public:

  Widget();
  
  // Value.
  
  void set(const image::Data *, const image::Color & min,
    const image::Color & max);
  
  // Widget.
  
  void draw();
  
  // Layout.
  
  void dirty();
  
private:

  const image::Data * _value;
  int _value_max;
  image::Color _min, _max;
};

//------------------------------------------------------------------------------
// Dialog

class Dialog : public ui::dialog::Base
{
public:
  
  Dialog();
  
  // Value.
  
  void set(const view::Widget *);
  void update(const view::Widget *);
  void del(const view::Widget *);

  // Widget.
  
  void show();
  void widget_del();
  
  // Layout.
  
  void update() { Base::update(); }
  
private:

  DJV_CALLBACK(Dialog, size_callback, int);
  DJV_CALLBACK(Dialog, close_callback, bool);
  
  void widget_update();
  
  const view::Widget * _view;
  image::gl::data::HISTOGRAM _size;
  image::Data _histogram;
  image::Color _min, _max;
  bool _update;
  histogram::Widget * _widget;
  ui::choice::Widget * _size_widget;
  ui::text::Display * _min_widget, * _max_widget;
};

//------------------------------------------------------------------------------
// Global

Dialog * dialog();

void del();

}
}

#endif

