/*------------------------------------------------------------------------------
 bin/djv_view/histogram_dialog.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "histogram.h"
#include <djv_ui/prefs.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/text.h>
#include <djv_ui/choice.h>
#include <djv_ui/app.h>

namespace djv_view {
namespace histogram
{

Dialog::Dialog() :
  Base("Histogram"),
  _view(0),
  _size(image::gl::data::HISTOGRAM_256),
  _update(false)
{
  // Widgets.
  
  _widget = new histogram::Widget;
  
  _size_widget = new ui::choice::Widget(image::gl::data::label_histogram());
  ui::label::Widget * size_label = new ui::label::Widget("Size:");
  
  _min_widget = new ui::text::Display;
  ui::label::Widget * min_label = new ui::label::Widget("Min:");
  _max_widget = new ui::text::Display;
  ui::label::Widget * max_label = new ui::label::Widget("Max:");
  
  ui::button::Push * close_button = new ui::button::Push("Clos&e");

  // Layout.
  
  ui::layout::group(List<ui::layout::Item *>() << min_label << max_label);
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  
  layout->add(_widget);
  layout->stretch(_widget);
  
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(min_label);
  h_layout->add(_min_widget);  
  h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(max_label);
  h_layout->add(_max_widget);
  
  h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(size_label);
  h_layout->add(_size_widget);
  h_layout->add_spacer(-1, true);
  h_layout->add(close_button);
  
  // Initialize.

  ui::prefs::Prefs prefs(ui::prefs::app(), "histogram");
  List<Str> tmp;
  if (prefs.get("size", &tmp)) str::serialize(&tmp, &_size,
    image::gl::data::label_histogram());

  widget_update();

  size(size_hint());
  close_button->take_focus();
  
  // Callbacks.
  
  _size_widget->sig.set(this, size_callback);
  close_button->sig.set(this, close_callback);
}

void Dialog::widget_del()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "histogram");
  prefs.set("size", str::serialize(_size, image::gl::data::label_histogram()));

  Base::widget_del();
}

void Dialog::set(const view::Widget * in)
{
  if (in == _view) return;
  _view = in;
  if (shown() && visible()) widget_update();
  else _update = true;
}

void Dialog::update(const view::Widget * in)
{
  if (in != _view || ! shown() || ! visible()) return;
  widget_update();
}

void Dialog::del(const view::Widget * in)
{
  if (in != _view) return;
  _view = 0;
}

void Dialog::show()
{
  if (_update) widget_update();
  ui::dialog::Base::show();
}

void Dialog::size_callback(int in)
{
  _size = image::gl::data::HISTOGRAM(in);
  widget_update();
}

void Dialog::close_callback(bool)
{
  hide();
}

void Dialog::widget_update()
{
  if (! _view) return;
  const image::data::Data * data = _view->get();
  if (! data) return;
  
  DJV_DBG_OBJ("histogram::Dialog::widget_update");
  
  callbacks(false);
  
  image::gl::data::Draw draw;
  draw.profile = _view->profile();
  draw.display = _view->display();
  draw.channel = _view->channel();
  try
  {
    image::gl::data::histogram(*data, _size, &_histogram, &_min, &_max, draw);
  }
  catch (Error in)
  {
    ui::app::global()->error(in);
  }
  
  _widget->set(&_histogram, _min, _max);
  _size_widget->set(_size);
  _min_widget->set(image::color::label(_min));
  _max_widget->set(image::color::label(_max));
  
  _update = false;
  
  callbacks(true);
  
  Base::update();
}

namespace { std::auto_ptr<Dialog> _dialog; }

Dialog * dialog()
{
  if (! _dialog.get()) _dialog = std::auto_ptr<Dialog>(new Dialog);
  return _dialog.get();
}

void del()
{
  if (_dialog.get()) _dialog->widget_del();
  _dialog.release();
}

}
}

