/*------------------------------------------------------------------------------
 bin/djv_view/prefs.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "prefs.h"
#include <djv_ui/prefs.h>
#include "cache.h"

namespace djv_view {
namespace prefs
{

File::File() :
  _seq(true),
  _proxy(0),
  _cache(false), _cache_size(cache::size_default()[0])
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "file");
  List<Str> tmp;
  if (prefs.get("seq", &tmp)) str::serialize(&tmp, &_seq);
  if (prefs.get("recent", &tmp)) str::serialize(&tmp, &_recent);
  if (prefs.get("proxy", &tmp)) str::serialize(&tmp, &_proxy);
  if (prefs.get("cache", &tmp)) str::serialize(&tmp, &_cache);
  if (prefs.get("cache_size", &tmp)) str::serialize(&tmp, &_cache_size);
}
  
File::~File()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "file");
  prefs.set("seq", str::serialize(_seq));
  prefs.set("recent", str::serialize(_recent));
  prefs.set("proxy", str::serialize(_proxy));
  prefs.set("cache", str::serialize(_cache));
  prefs.set("cache_size", str::serialize(_cache_size));
}

void File::seq(bool in)
{
  if (in == _seq) return;
  _seq = in;
  seq_sig.emit(_seq);
}

void File::recent(const Str & in)
{
  DJV_DBG_OBJ("prefs::File::recent");
  DJV_DBG_PRN("in = " << in);
  
  ui::recent(in, &_recent);
  recent_sig.emit(_recent);
}

void File::proxy(int in)
{
  if (in == _proxy) return;
  _proxy = in;
  proxy_sig.emit(_proxy);
}

void File::cache(bool in)
{
  if (in == _cache) return;
  _cache = in;
  if (! _cache) cache::global()->del();
  cache_sig.emit(_cache);
}

void File::cache_size(int in)
{
  if (in == _cache_size) return;
  _cache_size = in;
  cache::global()->max(_cache_size);
}

Window::Window() : _resize(true), _controls(true)
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "window");
  List<Str> tmp;
  if (prefs.get("resize", &tmp)) str::serialize(&tmp, &_resize);
  if (prefs.get("controls", &tmp)) str::serialize(&tmp, &_controls);
}

Window::~Window()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "window");
  prefs.set("resize", str::serialize(_resize));
  prefs.set("controls", str::serialize(_controls));
}

void Window::resize(bool in)
{
  if (in == _resize) return;
  _resize = in;
  resize_sig.emit(_resize);
}

void Window::controls(bool in)
{
  if (in == _controls) return;
  _controls = in;
  controls_sig.emit(_controls);
}

View::View() :
  _grid(view::GRID(0)), _grid_color(djv::image::Color(0.25)),
  _hud(false), _hud_foreground(djv::image::Color(1.0)),
    _hud_background(djv::image::Color(0.0)),
  _background(djv::image::Color(0.0))
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "view");
  List<Str> tmp;
  if (prefs.get("grid", &tmp)) str::serialize(&tmp, &_grid, view::label_grid());
  if (prefs.get("grid_color", &tmp)) djv::image::color::serialize(&tmp,
    &_grid_color);
  if (prefs.get("hud", &tmp)) str::serialize(&tmp, &_hud);
  if (prefs.get("hud_foreground", &tmp)) djv::image::color::serialize(&tmp,
    &_hud_foreground);
  if (prefs.get("hud_background", &tmp)) djv::image::color::serialize(&tmp,
    &_hud_background);
  if (prefs.get("background", &tmp)) djv::image::color::serialize(&tmp,
    &_background);
}

View::~View()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "view");
  prefs.set("grid", str::serialize(_grid, view::label_grid()));
  prefs.set("grid_color", djv::image::color::serialize(_grid_color));
  prefs.set("hud", str::serialize(_hud));
  prefs.set("hud_foreground", djv::image::color::serialize(_hud_foreground));
  prefs.set("hud_background", djv::image::color::serialize(_hud_background));
  prefs.set("background", djv::image::color::serialize(_background));
}

void View::grid(view::GRID in)
{
  if (in == _grid) return;
  _grid = in;
  grid_sig.emit(_grid);
}

void View::grid_color(const djv::image::Color & in)
{
  if (in == _grid_color) return;
  _grid_color = in;
  grid_color_sig.emit(_grid_color);
}

void View::hud(bool in)
{
  if (in == _hud) return;
  _hud = in;
  hud_sig.emit(_hud);
}

void View::hud_foreground(const djv::image::Color & in)
{
  if (in == _hud_foreground) return;
  _hud_foreground = in;
  hud_foreground_sig.emit(_hud_foreground);
}

void View::hud_background(const djv::image::Color & in)
{
  if (in == _hud_background) return;
  _hud_background = in;
  hud_background_sig.emit(_hud_background);
}

void View::background(const djv::image::Color & in)
{
  if (in == _background) return;
  _background = in;
  background_sig.emit(_background);
}

Image::Image() :
  _mirror_x(false), _mirror_y(false),
  _scale(ui::image::SCALE(0)),
  _rotate(ui::image::ROTATE(0)),
  _profile(true),
  _color(-1),
  _channel(ui::image::CHANNEL(0)),
  _filter(ui::image::FILTER(0))
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "image");
  List<Str> tmp;
  if (prefs.get("mirror_x", &tmp)) str::serialize(&tmp, &_mirror_x);
  if (prefs.get("mirror_y", &tmp)) str::serialize(&tmp, &_mirror_y);
  if (prefs.get("scale", &tmp))
    str::serialize(&tmp, &_scale, ui::image::label_scale());
  if (prefs.get("rotate", &tmp))
    str::serialize(&tmp, &_rotate, ui::image::label_rotate());
  
  if (prefs.get("profile", &tmp)) str::serialize(&tmp, &_profile);

  ui::prefs::Prefs color_prefs(prefs, "color");
  if (color_prefs.get("value", &tmp)) str::serialize(&tmp, &_color);
  int color_list_size = 0;
  if (color_prefs.get("size", &tmp))
    str::serialize(&tmp, &color_list_size);
  for (int i = 0; i < color_list_size; ++i)
    if (color_prefs.get(str::Format("%1").arg(i), &tmp))
    {
      Color value;
      serialize(&tmp, &value);
      value.lut_load();
      _color_list += value;
    }
  
  if (prefs.get("channel", &tmp))
    str::serialize(&tmp, &_channel, ui::image::label_channel());
  if (prefs.get("filter", &tmp))
    str::serialize(&tmp, &_filter, ui::image::label_filter());
}
  
Image::~Image()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "image");
  prefs.set("mirror_x", str::serialize(_mirror_x));
  prefs.set("mirror_y", str::serialize(_mirror_y));
  prefs.set("scale", str::serialize(_scale, ui::image::label_scale()));
  prefs.set("rotate", str::serialize(_rotate, ui::image::label_rotate()));
  
  prefs.set("profile", str::serialize(_profile));

  ui::prefs::Prefs color_prefs(prefs, "color");
  color_prefs.set("value", str::serialize(_color));
  color_prefs.set("size", str::serialize(int(_color_list.size())));
  for (uint i = 0; i < _color_list.size(); ++i)
    color_prefs.set(str::Format("%1").arg(i), serialize(_color_list[i]));
  
  prefs.set("channel", str::serialize(_channel, ui::image::label_channel()));
  prefs.set("filter", str::serialize(_filter, ui::image::label_filter()));
}

void Image::mirror_x(bool in)
{
  if (in == _mirror_x) return;
  _mirror_x = in;
  mirror_x_sig.emit(_mirror_x);
}

void Image::mirror_y(bool in)
{
  if (in == _mirror_y) return;
  _mirror_y = in;
  mirror_y_sig.emit(_mirror_y);
}

void Image::scale(ui::image::SCALE in)
{
  if (in == _scale) return;
  _scale = in;
  scale_sig.emit(_scale);
}

void Image::rotate(ui::image::ROTATE in)
{
  if (in == _rotate) return;
  _rotate = in;
  rotate_sig.emit(_rotate);
}

void Image::profile(bool in)
{
  if (in == _profile) return;
  _profile = in;
  profile_sig.emit(_profile);
}

void Image::color(int in)
{
  if (in == _color) return;
  _color = in;
  color_sig.emit(_color);
  color_value_sig.emit((_color != -1) ? _color_list[_color] : Color());
}

void Image::color_list(const List<Color> & in)
{
  _color_list = in;
  _color = math::min(_color, int(_color_list.size()) - 1);
  color_list_sig.emit(true);
}

List<Str> Image::color_name_list() const
{
  List<Str> out;
  for (uint i = 0; i < _color_list.size(); ++i)
    out += _color_list[i].name;
  return out;
}

void Image::channel(ui::image::CHANNEL in)
{
  if (in == _channel) return;
  _channel = in;
  channel_sig.emit(_channel);
}

void Image::filter(ui::image::FILTER in)
{
  if (in == _filter) return;
  _filter = in;
  filter_sig.emit(_filter);
}

Playback::Playback() :
  _start(true),
  _every_frame(true)
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "playback");
  List<Str> tmp;
  if (prefs.get("start", &tmp)) str::serialize(&tmp, &_start);
  if (prefs.get("every_frame", &tmp)) str::serialize(&tmp, &_every_frame);
}

Playback::~Playback()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "playback");
  prefs.set("start", str::serialize(_start));
  prefs.set("every_frame", str::serialize(_every_frame));
}

void Playback::start(bool in) { _start = in; }

void Playback::every_frame(bool in)
{
  if (in == _every_frame) return;
  _every_frame = in;
  every_frame_sig.emit(_every_frame);
}

namespace
{
std::auto_ptr<File> _file;
std::auto_ptr<Window> _window;
std::auto_ptr<View> _view;
std::auto_ptr<Image> _image;
std::auto_ptr<Playback> _playback;
}

File * file() { return _file.get(); }
Window * window() { return _window.get(); }
View * view() { return _view.get(); }
Image * image() { return _image.get(); }
Playback * playback() { return _playback.get(); }

void init()
{
  _file = std::auto_ptr<File>(new File);
  _window = std::auto_ptr<Window>(new Window);
  _view = std::auto_ptr<View>(new View);
  _image = std::auto_ptr<Image>(new Image);
  _playback = std::auto_ptr<Playback>(new Playback);
}

void del()
{
  _file.reset();
  _window.reset();
  _view.reset();
  _image.reset();
  _playback.reset();
}

}
}

