/*------------------------------------------------------------------------------
 bin/djv_view/prefs.h
 
 Preferences
 -----------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_PREFS_H
#define DJV_VIEW_PREFS_H

#include "view.h"

namespace djv_view {
namespace prefs
{

//------------------------------------------------------------------------------
// File Preferences

class File
{
public:

  File();
  ~File();
  
  void seq(bool);
  bool seq() const { return _seq; }
  ui::Signal<bool> seq_sig;

  void recent(const Str &);
  const List<Str> & recent() const { return _recent; }
  ui::Signal<const List<Str> &> recent_sig;

  void proxy(int);
  int proxy() const { return _proxy; }
  ui::Signal<int> proxy_sig;

  void cache(bool);
  void cache_size(int);
  bool cache() const { return _cache; }
  int cache_size() const { return _cache_size; }
  ui::Signal<bool> cache_sig;

private:

  bool _seq;
  List<Str> _recent;
  int _proxy;
  bool _cache;
  int _cache_size;
};

//------------------------------------------------------------------------------
// Window Preferences

class Window
{
public:

  Window();
  ~Window();

  void resize(bool);
  bool resize() const { return _resize; }
  ui::Signal<bool> resize_sig;

  void controls(bool);
  bool controls() const { return _controls; }
  ui::Signal<bool> controls_sig;

private:

  bool _resize;
  bool _controls;
};

//------------------------------------------------------------------------------
// View Preferences

class View
{
public:

  View();
  ~View();

  void grid(view::GRID);
  void grid_color(const image::Color &);
  view::GRID grid() const { return _grid; }
  const image::Color & grid_color() const { return _grid_color; }
  ui::Signal<view::GRID> grid_sig;
  ui::Signal<const image::Color &> grid_color_sig;

  void hud(bool);
  void hud_foreground(const image::Color &);
  void hud_background(const image::Color &);
  bool hud() const { return _hud; }
  const image::Color & hud_foreground() const { return _hud_foreground; }
  const image::Color & hud_background() const { return _hud_background; }
  ui::Signal<bool> hud_sig;
  ui::Signal<const image::Color &> hud_foreground_sig;
  ui::Signal<const image::Color &> hud_background_sig;

  void background(const image::Color &);
  const image::Color & background() const { return _background; }
  ui::Signal<const image::Color &> background_sig;

private:

  view::GRID _grid;
  image::Color _grid_color;
  bool _hud;
  image::Color _hud_foreground, _hud_background;
  image::Color _background;
};

//------------------------------------------------------------------------------
// Image Preferences

class Image
{
public:

  Image();
  ~Image();

  void mirror_x(bool);
  void mirror_y(bool);
  bool mirror_x() const { return _mirror_x; }
  bool mirror_y() const { return _mirror_y; }
  ui::Signal<bool> mirror_x_sig, mirror_y_sig;

  void scale(ui::image::SCALE);
  ui::image::SCALE scale() const { return _scale; }
  ui::Signal<ui::image::SCALE> scale_sig;

  void rotate(ui::image::ROTATE);
  ui::image::ROTATE rotate() const { return _rotate; }
  ui::Signal<ui::image::ROTATE> rotate_sig;
  
  void profile(bool);
  bool profile() const { return _profile; }
  ui::Signal<bool> profile_sig;

  void color(int);
  void color_list(const List<Color> &);
  int color() const { return _color; }
  Color color_value() const
    { return _color != -1 ? _color_list[_color] : Color(); }
  const List<Color> & color_list() const { return _color_list; }
  List<Str> color_name_list() const;
  ui::Signal<int> color_sig;
  ui::Signal<const Color &> color_value_sig;
  ui::Signal<bool> color_list_sig;

  void channel(ui::image::CHANNEL);
  ui::image::CHANNEL channel() const { return _channel; }
  ui::Signal<ui::image::CHANNEL> channel_sig;

  void filter(ui::image::FILTER);
  ui::image::FILTER filter() const { return _filter; }
  ui::Signal<ui::image::FILTER> filter_sig;

private:

  bool _mirror_x, _mirror_y;
  ui::image::SCALE _scale;
  ui::image::ROTATE _rotate;
  bool _profile;
  int _color;
  List<Color> _color_list;
  ui::image::CHANNEL _channel;
  ui::image::FILTER _filter;
};

//------------------------------------------------------------------------------
// Playback Preferences

class Playback
{
public:

  Playback();
  ~Playback();

  void start(bool);
  bool start() const { return _start; }

  void every_frame(bool);
  bool every_frame() const { return _every_frame; }
  ui::Signal<bool> every_frame_sig;

private:

  bool _start, _every_frame;
};

//------------------------------------------------------------------------------
// Global

File * file();
Window * window();
View * view();
Image * image();
Playback * playback();

void init();
void del();

}
}

#endif

