/*------------------------------------------------------------------------------
 bin/djv_vlut/base.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "base.h"
#include <djv_image/io.h>
#include <djv_ui/prefs.h>
#include <djv_ui/app.h>

namespace djv_vlut
{

void Color::lut_load()
{
  DJV_DBG_OBJ("Color::lut_load");
  
  lut = image::Image();
  if (lut_file.size())
    try
    {
      lut = *image::io::load(lut_file)->get();
    }
    catch (Error in)
    {
      ui::app::global()->error(str::Format("%1: %2").
        arg(in).arg(File(lut_file).name()));
    }
}

Color_List::Color_List()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "color_list");
  List<Str> tmp;
  int list_size = 0;
  if (prefs.get("size", &tmp))
    str::serialize(&tmp, &list_size);
  for (int i = 0; i < list_size; ++i)
    if (prefs.get(str::Format("%1").arg(i), &tmp))
    {
      Color value;
      serialize(&tmp, &value);
      value.lut_load();
      _list += value;
    }
}

Color_List::~Color_List()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "color_list");
  prefs.set("size", str::serialize(int(_list.size())));
  for (uint i = 0; i < _list.size(); ++i)
    prefs.set(str::Format("%1").arg(i), serialize(_list[i]));
}

void Color_List::set(const List<Color> & in)
{
  _list = in;
  sig.emit(true);
}

List<Str> Color_List::name_list() const
{
  List<Str> out;
  for (uint i = 0; i < _list.size(); ++i)
    out += _list[i].name;
  return out;
}

bool operator == (const Color & a, const Color & b)
{ return (const ui::image::Display &)a == (const ui::image::Display &)b &&
  a.lut_file == b.lut_file; }

bool operator != (const Color & a, const Color & b) { return ! (a == b); }

bool serialize(List<Str> * in, Color * out)
{
  if (! str::serialize(in, &out->name)) return false;
  if (! str::serialize(in, &out->lut_file)) return false;
  if (! image::gl::data::serialize(in, out)) return false;
  return true;
}

List<Str> serialize(const Color & in)
{
  return List<Str>() << str::serialize(in.name) <<
    str::serialize(in.lut_file) << image::gl::data::serialize(in);
}

namespace { std::auto_ptr<Color_List> _color_list; }

Color_List * color_list()
{
  if (! _color_list.get())
    _color_list = std::auto_ptr<Color_List>(new Color_List);
  return _color_list.get();
}

void color_list_del() { _color_list.reset(); }

}

