/*------------------------------------------------------------------------------
 bin/djv_vlut/window.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "window.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/menu.h>
#include <djv_ui/messages_dialog.h>
#include <djv_ui/prefs_dialog.h>
#include <djv_ui/about_dialog.h>
#include <djv_ui/image_io.h>
#include <djv_ui/app.h>
#include <FL/Fl.H>

namespace djv_vlut {
namespace window
{

Widget::Widget() : ui::window::Widget(ui::app::global()->name())
{
  // Widgets.
  
  _widget = new widget::Widget;
  _menu = new ui::menu::Widget;
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(0);
  layout->margin(0);
  layout->add(_menu);
  layout->add(_widget);
  
  // Initialize.
  
  value_update();
  menu_update();
  
  size(size_hint());
  Fl::focus(this);
  
  // Callbacks.
 
  _widget->sig.set(this, color_callback);
  color_list()->sig.set(this, color_list_callback);
  ui::image_io::sig.set(this, image_io_callback);
  close_sig.set(this, close_callback);
}

void Widget::set(const Color & in)
{
  if (in == _color) return;
  _color = in;
  value_update();
  sig.emit(_color);
}

void Widget::color_callback(const Color & in) { set(in); }

void Widget::_color_reset_callback() { set(Color()); }

void Widget::color_add_callback(const Str & in)
{
  List<Color> list = color_list()->get();
  Color tmp = _color;
  tmp.name = in;
  list += tmp;
  color_list()->set(list);
}

void Widget::_color_add_callback()
{
  ui::dialog::input("Add color setting:", "Color", this, color_add_callback);
}

void Widget::color_del_callback(int in)
{
  List<Color> list = color_list()->get();
  List<Color>::iterator i = list.begin();
  for (int j = 0; j < in && i != list.end(); ++j, ++i) ;
  list.erase(i);
  color_list()->set(list);
}

void Widget::_color_del_callback()
{
  const List<Str> list = color_list()->name_list();
  if (! list.size()) return;
  ui::dialog::choice("Delete color setting:", list, this, color_del_callback);
}

void Widget::color_del_all_callback(bool)
  { color_list()->set(List<Color>()); }

void Widget::_color_del_all_callback()
{
  const List<Str> list = color_list()->name_list();
  if (! list.size()) return;
  ui::dialog::question("Delete all color settings?", this,
    color_del_all_callback);
}

void Widget::color_list_callback(bool) { menu_update(); }

void Widget::_color_list_callback()
{
  color_callback(color_list()->get()[list::find(_menu->menu(), _menu_color)]);
}

void Widget::_messages_callback() { ui::messages_dialog::global()->show(); }
void Widget::_prefs_callback() { ui::prefs_dialog::global()->show(); }
void Widget::_about_callback() { ui::about_dialog::global()->show(); }
void Widget::_exit_callback() { ui::app::global()->exit(); }

void Widget::image_io_callback(image::io::Plugin *) { value_update(); }

void Widget::close_callback(bool) { ui::app::global()->exit(); }

void Widget::value_update()
{
  callbacks(false);
  
  _widget->set(_color);
  
  callbacks(true);
}

void Widget::menu_update()
{
  callbacks(false);
  
  _menu->menu_del();
  
  _menu->menu_add("&File", 0, 0, 0, ui::menu::SUB_MENU);
  _menu->menu_add("&Messages", 0, _messages_callback, this);
  _menu->menu_add("&Preferences", 0, _prefs_callback, this);
  _menu->menu_add("&About", 0, _about_callback, this, ui::menu::DIVIDER);
  _menu->menu_add("Exit", FL_ALT + 'q', _exit_callback, this);
  _menu->menu_end();

  _menu->menu_add("&Color", 0, 0, 0, ui::menu::SUB_MENU);
  _menu->menu_add("&Reset", FL_ALT + '`', _color_reset_callback, this,
    ui::menu::DIVIDER);
  _menu->menu_add("&Add", 0, _color_add_callback, this);
  _menu->menu_add("&Delete", 0, _color_del_callback, this);
  _menu->menu_add("Delete A&ll", 0, _color_del_all_callback, this,
    ui::menu::DIVIDER);
  _menu_color.clear();
  List<Str> tmp = color_list()->name_list();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_color += _menu->menu_add(
      ui::menu::list_name(tmp[i], i), i < 9 ? (FL_ALT + '1' + i) : 0,
      _color_list_callback, this);
  _menu->menu_end();
  _menu->menu_end();

  _menu->menu_end();
  
  callbacks(true);
}

}
}

