/*------------------------------------------------------------------------------
 lib/djv_base/app.h
 
 Command Line Application
 ------------------------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_APP_H
#define DJV_APP_H

#include <djv_base/base.h>

namespace djv {
namespace app
{

//------------------------------------------------------------------------------
// Application

enum EXIT { EXIT_DEFAULT, EXIT_ERROR, EXIT_HELP, EXIT_INFO, EXIT_ABOUT };

class App
{
public:

  App(const Str & name, int & argc, char ** argv);
  virtual ~App();
  
  // Execution.

  virtual int exec();
  
  virtual void exit(EXIT);
  EXIT exit() const { return _exit; }

  // Command line.

  const Str & cmdln_name() const { return _cmdln_name; }
  
  List<Str> cmdln_list;
  
  // Information.
  
  const Str & name() const { return _name; }
  
  virtual Str info() const;
  virtual Str about() const;
  virtual Str help() const;
  
  // Messages.

  virtual void message(const Str &) const;
  virtual void error(const Str &) const;
  virtual void error(const Error &) const;
  virtual void print(const Str &, bool newline = true) const;
  virtual void separator() const;

private:

  EXIT _exit;
  Str _name;
  Str _cmdln_name;
  bool _endline, _separator;
};

//------------------------------------------------------------------------------
// Global

App * global();

}
}

#endif

