/*------------------------------------------------------------------------------
 lib/djv_base/box2_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv {
namespace box
{

#define _BOX2_INIT \
  x(Box<T, 2>::position.e[0]), \
  y(Box<T, 2>::position.e[1]), \
  w(Box<T, 2>::size.e[0]), \
  h(Box<T, 2>::size.e[1])

template<typename T>
inline Box2<T>::Box2() : _BOX2_INIT
{}

template<typename T>
inline Box2<T>::Box2(const Box2<T> & in) : _BOX2_INIT
{ this->position = in.position; this->size = in.size; }

template<typename T>
inline Box2<T>::Box2(const Box<T, 2> & in) : _BOX2_INIT
{ this->position = in.position; this->size = in.size; }

template<typename T>
inline Box2<T>::Box2(T x, T y, T w, T h) : _BOX2_INIT
{ this->x = x; this->y = y; this->w = w; this->h = h; }

template<typename T>
inline Box2<T>::Box2(T w, T h) : _BOX2_INIT
{ this->w = w; this->h = h; }

template<typename T>
inline Box2<T>::Box2(const Vec<T, 2> & position, const Vec<T, 2> & size) :
  _BOX2_INIT
{ this->position = position; this->size = size; }

template<typename T>
inline Box2<T>::Box2(const Vec<T, 2> & size) : _BOX2_INIT
{ this->size = size; }

template<typename T>
inline void Box2<T>::x1(const T & in) { w = in - x; }
template<typename T>
inline void Box2<T>::y1(const T & in) { h = in - y; }

template<>
inline void Box2<int>::x1(const int & in) { w = in - x + 1; }
template<>
inline void Box2<int>::y1(const int & in) { h = in - y + 1; }

template<typename T>
inline T Box2<T>::x1() const { return x + w; }
template<typename T>
inline T Box2<T>::y1() const { return y + h; }

template<>
inline int Box2<int>::x1() const { return x + w - 1; }
template<>
inline int Box2<int>::y1() const { return y + h - 1; }

template<typename T>
inline void Box2<T>::position1(const Vec<T, 2> & in)
{
  w = in.e[0] - x;
  h = in.e[1] - y;
}

template<>
inline void Box2<int>::position1(const Vec<int, 2> & in)
{
  w = in.e[0] - x + 1;
  h = in.e[1] - y + 1;
}

template<typename T>
inline Vec<T, 2> Box2<T>::position1() const
{
  Vec<T, 2> out;
  out.e[0] = x + w;
  out.e[1] = y + h;
  return out;
}

template<>
inline Vec<int, 2> Box2<int>::position1() const
{
  Vec<int, 2> out;
  out.e[0] = x + w - 1;
  out.e[1] = y + h - 1;
  return out;
}

}
}

