/*------------------------------------------------------------------------------
 lib/djv_base/box_fnc_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/math.h>

namespace djv {
namespace box
{

template<typename T, int D>
inline Box<T, D> swap(const Box<T, D> & in)
{
  return Box<T, D>(vec::swap(in.position), vec::swap(in.size));
}

template<typename T>
inline bool intersect(const Box2<T> & a, const Vec2<T> & b)
{
  if (! math::intersect(b.x, a.x, a.x1())) return false;
  if (! math::intersect(b.y, a.y, a.y1())) return false;
  return true;
}

template<typename T>
inline Box2<T> intersect(const Box2<T> & a, const Box2<T> & b)
{
  Box2<T> out;
  T x1 = T(0), y1 = T(0);
  math::intersect(a.x, a.x1(), b.x, b.x1(), &out.x, &x1);
  math::intersect(a.y, a.y1(), b.y, b.y1(), &out.y, &y1);
  out.position1(Vec2<T>(x1, y1));
  return out;
}

template<typename T>
inline Box2<T> bound(const Box2<T> & a, const Box2<T> & b)
{
  Box2<T> out;
  T x1 = T(0), y1 = T(0);
  math::bound(a.x, a.x1(), b.x, b.x1(), &out.x, &x1);
  math::bound(a.y, a.y1(), b.y, b.y1(), &out.y, &y1);
  out.position1(Vec2<T>(x1, y1));
  return out;
}

template<typename T>
inline Box2<T> expand(const Box2<T> & a, const Vec2<T> & b)
{
  return Box2<T>(a.position - b, a.size + b * T(2));
}

template<typename T, int D>
inline bool size_valid(const Box<T, D> & in)
{
  return vec::size_valid(in.size);
}

template <typename T, int D>
inline Box<T, D> operator + (const Box<T, D> & a, T b)
{
  return Box<T, D>(a.position + b, a.size + b);
}

template <typename T, int D>
inline Box<T, D> operator - (const Box<T, D> & a, T b)
{
  return Box<T, D>(a.position - b, a.size - b);
}

template <typename T, int D>
inline Box<T, D> operator * (const Box<T, D> & a, T b)
{
  return Box<T, D>(a.position * b, a.size * b);
}

template <typename T, int D>
inline Box<T, D> operator / (const Box<T, D> & a, T b)
{
  return Box<T, D>(a.position / b, a.size / b);
}

template <typename T, int D>
inline bool operator == (const Box<T, D> & a, const Box<T, D> & b)
{
  return a.position == b.position && a.size == b.size;
}

template <typename T, int D>
inline bool operator != (const Box<T, D> & a, const Box<T, D> & b)
{ return ! (a == b); }

template <int D>
inline Box<int, D> to_int(const Box<double, D> & in)
{
  Box<int, D> out;
  out.position = vec::to_int(in.position);
  out.size = vec::to_int(in.size);
  return out;
}

template <int D>
inline Box<double, D> to_float(const Box<int, D> & in)
{
  Box<double, D> out;
  out.position = vec::to_float(in.position);
  out.size = vec::to_float(in.size);
  return out;
}

template <typename T, int D>
inline bool serialize(List<Str> * in, Box<T, D> * out)
{
  if (! vec::serialize(in, &out->position)) return false;
  if (! vec::serialize(in, &out->size)) return false;
  return true;
}

template <typename T, int D>
inline List<Str> serialize(const Box<T, D> & in)
{
  return List<Str>() << vec::serialize(in.position)<< vec::serialize(in.size);
}

template <typename T, int D>
inline Dbg & operator << (Dbg & dbg, const Box<T, D> & in)
{ return dbg << serialize(in); }

}
}
