/*------------------------------------------------------------------------------
 lib/djv_base/vec3_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv {
namespace vec
{

#define _VEC3_INIT \
  x(Vec<T, 3>::e[0]), \
  y(Vec<T, 3>::e[1]), \
  y(Vec<T, 3>::e[2])

template<typename T>
inline Vec3<T>::Vec3() : _VEC3_INIT {}

template<typename T>
inline Vec3<T>::Vec3(const Vec3<T> & in) : _VEC3_INIT
{ x = in.x; y = in.y; z = in.z; }

template<typename T>
inline Vec3<T>::Vec3(const Vec<T, 3> & in) : _VEC3_INIT
{ x = in.e[0]; y = in.e[1]; z = in.e[2]; }

template<typename T>
inline Vec3<T>::Vec3(T _x, T _y, T _z) : _VEC3_INIT { x = _x; y = _y; z = _z; }

template<typename T>
inline Vec3<T>::Vec3(T in) : _VEC3_INIT { x = y = z = in; }

template<>
inline void Vec<int, 3>::set(const Vec<int, 3> & in)
{
  Vec<int, 3>::e[0] = in.e[0];
  Vec<int, 3>::e[1] = in.e[1];
  Vec<int, 3>::e[2] = in.e[2];
}

template<>
inline void Vec<int, 3>::set(int in)
{
  e[0] = e[1] = e[2] = in;
}

template<>
inline void Vec<int, 3>::zero()
{
  e[0] = e[1] = e[2] = 0;
}

template<>
inline Vec<int, 3> & Vec<int, 3>::operator = (const Vec<int, 3> & in)
{
  Vec<int, 3>::e[0] = in.e[0];
  Vec<int, 3>::e[1] = in.e[1];
  Vec<int, 3>::e[2] = in.e[2];
  return *this;
}

#define _VEC3_OP(IN) \
  template<> \
  inline Vec<int, 3> & Vec<int, 3>::operator IN (const Vec<int, 3> & in) \
  { \
    Vec<int, 3>::e[0] IN in.e[0]; \
    Vec<int, 3>::e[1] IN in.e[1]; \
    Vec<int, 3>::e[2] IN in.e[2]; \
    return *this; \
  }
_VEC3_OP(+=)
_VEC3_OP(-=)
_VEC3_OP(/=)
_VEC3_OP(*=)

#define _VEC3_OP3(IN) \
  template<> \
  inline Vec<int, 3> & Vec<int, 3>::operator IN (int in) \
  { \
    Vec<int, 3>::e[0] IN in; \
    Vec<int, 3>::e[1] IN in; \
    Vec<int, 3>::e[2] IN in; \
    return *this; \
  }
_VEC3_OP3(+=)
_VEC3_OP3(-=)
_VEC3_OP3(/=)
_VEC3_OP3(*=)

}
}

