/*------------------------------------------------------------------------------
 lib/djv_image/io_cineon_save.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/io_cineon_save.cxx"

#include "io_cineon.h"

namespace djv {
namespace image {
namespace io {
namespace cineon
{

Plugin * Save::copy() const
{ Save * out = new Save; out->_options = _options; return out; }

namespace
{
const Str _help =
"* cineon color (value) - Output color space: %1. Default: %2.\n"
"* cineon print (black) (white) (gamma) - Print values. Default: %3.\n";
}

Str Save::help() const
{
  return str::Format(_help).
    arg(dpx::label_color(), ", ").
    arg(str::serialize(_options.color, dpx::label_color())).
    arg(dpx::serialize(_options.print));
}

bool Save::option(const Str & in, List<Str> * data)
{
  if (str::compare(in, option_list()[OPTION_COLOR]))
    return str::serialize(data, &_options.color, dpx::label_color());
  else if (str::compare(in, option_list()[OPTION_PRINT]))
    return dpx::serialize(data, &_options.print);
  return false;
}

List<Str> Save::option(const Str & in) const
{
  if (str::compare(in, option_list()[OPTION_COLOR]))
    return str::serialize(_options.color, dpx::label_color());
  else if (str::compare(in, option_list()[OPTION_PRINT]))
    return dpx::serialize(_options.print);
  return List<Str>();
}

const List<Str> & Save::option_list() const { return label_save_option(); }

void Save::open(const File & in, const Info & info, const Tag &, const Time &)
  throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::open");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("info = " << info);
  
  _file = in;
  
  _info = Info();
  _info.size = info.size;
  _info.pixel = pixel::RGB_U10;
  _info.mirror_y = true;
  _info.endian = mem::MSB;
  _info.align = 4;
  DJV_DBG_PRN("info = " << _info);
  
  _image.set(_info);
}

void Save::_open(const File & in, const Tag & tag) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv::io::WRITE);
  
  header_save(&_io, in, _info, tag, _options.color);
}

void Save::set(const Image & in, int frame) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::set");
  DJV_DBG_PRN("in = " << in);
  
  // Setup.
  
  Profile profile;
  if (dpx::COLOR_PRINT == _options.color || dpx::COLOR_AUTO == _options.color)
  {
    DJV_DBG_PRN("profile");
    profile.type = profile::LUT;
    profile.lut = dpx::print_lut(_options.print);
  }
  
  // Open.
  
  _open(_file.get(frame), in.tag);
  
  // Write.
  
  const Data * p = &in;
  if (in.info() != _info || in.profile.type != profile::RAW)
  {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    gl::data::Draw draw;
    draw.profile = profile;
    gl::data::copy(*p, &_image, draw);
    p = &_image;
  }
  
  _io.set(p->data(), p->data_bytes());

  header_save_update(&_io);
  _io.close();
}

}
}
}
}

