/*------------------------------------------------------------------------------
 lib/djv_ui/_file_browser.h
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef _FILE_BROWSER_H
#define _FILE_BROWSER_H

#include <djv_base/file.h>
#include <djv_image/io.h>
#include <djv_ui/file_browser.h>
#include <FL/Fl_Browser_.H>
#include <list>

namespace djv {
namespace ui {
namespace file_browser
{

class Item
{
public:

  Item();
  ~Item();

  void set(const File &, bool odd);
  const File & get() const { return _file; }
  bool odd() const { return _odd; }
  int height() const { return _height; }

  Fl_Image * image(int thumbnail) const;
  const djv::image::Info & image_info() const { return _image_info; }
  const djv::image::Time & image_time() const { return _image_time; }
  bool image_valid() const { return _image_valid; }
  bool image_init() const { return _image_init; }

  Item * prev, * next;

private:
  
  File _file;
  bool _odd;
  int _height;
  Fl_Image * _image;
  djv::image::Info _image_info;
  djv::image::Time _image_time;
  bool _image_valid, _image_init;
};

class Browser : public Fl_Browser_, public widget::Base
{
public:

  Browser();
  ~Browser();
  
  // Value.
  
  void set(const List<File> &, bool images);
  File get() const;
  Signal<const File &> sig;
  Signal<const File &> action_sig;
  
  // Columns.

  void column_list(const List<int> &);
  
  // Widget.
  
  int handle(int);
  void draw();

  void * item_first() const
    { return _item_list.size() ? (void *)&_item_list.front() : 0; }
  void * item_next(void * in) const
    { return in ? static_cast<file_browser::Item *>(in)->next : 0; }
  void * item_prev(void * in) const
    { return in ? static_cast<file_browser::Item *>(in)->prev : 0; }
  int item_height(void *) const;
  int item_width(void *) const { return 0; }
  void item_draw(void *, int, int, int, int) const;

  // Layout.
  
  void dirty();
  void geom(const Box2i &);
  const Box2i & geom() const { return widget::Base::geom(); }
  void resize(int x, int y, int w, int h) { geom(Box2i(x, y, w, h)); }
  int visible() const { return Fl_Browser_::visible(); }
  
  // Debugging.

  Str dbg_name() const { return "djv::ui::file_browser::Browser"; }

private:
  
  void del();
  
  DJV_FL_WIDGET_CALLBACK(Browser, widget_callback);
  DJV_FL_CALLBACK(Browser, image_callback);

  
  List<file_browser::Item> _item_list;
  int _column_list [Dialog::sort_size];
  int _size_text, _size_icon, _size_thumbnail, _margin;
  bool _images;
  std::list<file_browser::Item *> _images_work_list;
};

List<int> columns_default_size_list();

}
}
}

#endif

