/*------------------------------------------------------------------------------
 lib/djv_ui/color.h
 
 Color Widgets
 -------------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_COLOR_H
#define DJV_UI_COLOR_H

#include <djv_image/color.h>
#include <djv_ui/widget.h>
#include <djv_ui/group.h>

namespace djv {
namespace ui {
namespace color
{

//------------------------------------------------------------------------------
// Color Swatch

class Swatch : public group::Frame
{
public:

  Swatch();
  virtual ~Swatch() {}

  // Value.
  
  void set(const djv::image::Color &);
  const djv::image::Color & get() const { return _value; }
  Signal<const djv::image::Color &> sig;
  
  // Widget.

  virtual void draw();
  
  // Layout.

  virtual void dirty();
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::color::Swatch"; }
  
private:

  djv::image::Color _value;
};

//------------------------------------------------------------------------------
// Color Widget

class Widget : public widget::Widget
{
public:

  Widget();
  virtual ~Widget() {}

  // Value.
  
  void set(const djv::image::Color &);
  const djv::image::Color & get() const { return _value; }
  Signal<const djv::image::Color &> sig;
  
  // Float range.
  
  void float_range(double min, double max);
  double float_min() const { return _float_min; }
  double float_max() const { return _float_max; }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::color::Widget"; }
  
private:

  DJV_CALLBACK(Widget, int_callback, int);
  DJV_CALLBACK(Widget, float_callback, double);
  DJV_CALLBACK(Widget, pixel_callback, djv::image::PIXEL);
  
  void widget_update();
  void value_update();

  djv::image::Color _value;
  double _float_min, _float_max;
  num::Int_Slider * _int_widget [djv::image::pixel::channels_max];
  num::Float_Slider * _float_widget [djv::image::pixel::channels_max];
  pixel::Widget * _pixel_widget;
};

}
}
}

#endif

