/*------------------------------------------------------------------------------
 lib/djv_ui/dialog.h
 
 Dialogs
 -------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_DIALOG_H
#define DJV_UI_DIALOG_H

#include <djv_ui/window.h>

namespace djv {
namespace ui {
namespace dialog
{

//------------------------------------------------------------------------------
// Dialog Base

class Base : public window::Widget
{
public:
  
  Base(const Str & title = Str());
  virtual ~Base() = 0;
};

//------------------------------------------------------------------------------
// Message Dialog Base

class Message_Base : public Base
{
public:

  Message_Base(const Str & title = Str());
  virtual ~Message_Base() = 0;
  
  // Layout.
  
  virtual void dirty();
  virtual const Vec2i & size_min() const { return _size_min; }
  
private:

  Vec2i _size_min;
};

//------------------------------------------------------------------------------
// Message Dialog

class Message : public Message_Base
{
public:
  
  Message(const Str & label = Str());
  
  // Label.
 
  void label(const Str &);
  Str label() const;

private:

  DJV_CALLBACK(Message, close_callback, bool);

  label::Widget * _widget;
};

//------------------------------------------------------------------------------
// Question Dialog

class Question : public Message_Base
{
public:
  
  Question(const Str & label = Str());
  
  // Value.
  
  Signal<bool> sig;

  // Label.

  void label(const Str &);
  Str label() const;
  
private:

  DJV_CALLBACK(Question, ok_callback, bool);
  DJV_CALLBACK(Question, close_callback, bool);

  label::Widget * _widget;
};

//------------------------------------------------------------------------------
// Input Dialog

class Input : public Message_Base
{
public:
  
  Input(const Str & label = Str(), const Str & value = Str());
  
  // Value.

  void set(const Str &);
  Str get() const;
  Signal<const Str &> sig;
  
  // Label.
  
  void label(const Str &);
  Str label() const;

private:

  DJV_CALLBACK(Input, widget_callback, const Str &);
  DJV_CALLBACK(Input, ok_callback, bool);
  DJV_CALLBACK(Input, close_callback, bool);

  label::Widget * _label;
  text::Edit * _widget;
};

//------------------------------------------------------------------------------
// Choice Dialog

class Choice : public Message_Base
{
public:
  
  Choice(const Str & label = Str(), const List<Str> & list = List<Str>());
  
  // Value.

  Signal<int> sig;
  
  // List.

  void list(const List<Str> &);
  List<Str> list() const;
  
  // Label.
  
  void label(const Str &);
  Str label() const;

private:

  DJV_CALLBACK(Choice, ok_callback, bool);
  DJV_CALLBACK(Choice, close_callback, bool);

  choice::Widget * _widget;
  label::Widget * _label;
};

//------------------------------------------------------------------------------
// Global

void message(const Str &);

void question(const Str &, Callback_Base *, Signal<bool>::Callback *);
void question_del(Callback_Base *);

void input(const Str & label, const Str & value,
  Callback_Base *, Signal<const Str &>::Callback *);
void input_del(Callback_Base *);

void choice(const Str & label, const List<Str> & list,
  Callback_Base *, Signal<int>::Callback *);
void choice_del(Callback_Base *);

void del();

}
}
}

#endif

