/*------------------------------------------------------------------------------
 lib/djv_ui/dialog_fnc.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "dialog.h"

namespace djv {
namespace ui {
namespace dialog
{

namespace
{
std::auto_ptr<Message> _message;
std::auto_ptr<Question> _question;
std::auto_ptr<Input> _input;
std::auto_ptr<Choice> _choice;
}

void message(const Str & label)
{
  if (! _message.get()) _message = std::auto_ptr<Message>(new Message);
  _message->label(label);
  _message->show();
}

void question(const Str & label, Callback_Base * in,
  Signal<bool>::Callback * callback)
{
  if (! _question.get()) _question = std::auto_ptr<Question>(new Question);
  _question->label(label);
  _question->sig.del();
  _question->sig.set(in, callback);
  _question->show();
}

void question_del(Callback_Base * in)
{
  if (! _question.get()) return;
  _question->sig.del(in);
}

void input(const Str & label, const Str & value,
  Callback_Base * in, Signal<const Str &>::Callback * callback)
{
  if (! _input.get()) _input = std::auto_ptr<Input>(new Input);
  _input->set(value);
  _input->label(label);
  _input->sig.del();
  _input->sig.set(in, callback);
  _input->show();
}

void input_del(Callback_Base * in)
{
  if (! _input.get()) return;
  _input->sig.del(in);
}

void choice(const Str & label, const List<Str> & list,
  Callback_Base * in, Signal<int>::Callback * callback)
{
  if (! _choice.get()) _choice = std::auto_ptr<Choice>(new Choice);
  _choice->list(list);
  _choice->label(label);
  _choice->sig.del();
  _choice->sig.set(in, callback);
  _choice->show();
}

void choice_del(Callback_Base * in)
{
  if (! _choice.get()) return;
  _choice->sig.del(in);
}

void del()
{
  if (_message.get()) _message->widget_del();
  if (_question.get()) _question->widget_del();
  if (_input.get()) _input->widget_del();
  if (_choice.get()) _choice->widget_del();
  _message.release();
  _question.release();
  _input.release();
  _choice.release();
}

}
}
}

