/*------------------------------------------------------------------------------
 lib/djv_ui/group.h
 
 Group Widgets
 -------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_GROUP_H
#define DJV_UI_GROUP_H

#include <djv_ui/widget.h>

namespace djv {
namespace ui {
namespace group
{

//------------------------------------------------------------------------------
// Frame Widget

class Frame : public widget::Widget
{
public:
  
  Frame(const Fl_Boxtype = FL_THIN_UP_FRAME);
  virtual ~Frame() {}
  
  // Frame.
  
  void frame(const Fl_Boxtype);
  const Fl_Boxtype & frame() const { return _frame; }

  void edges(bool left, bool right, bool top, bool bottom);
  const bool * edges() const { return _edges; }
  
  // Widget.
  
  virtual void draw();
  
  // Layout.

  int frame_size() const;
  Box2i frame_geom() const;
  
  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return widget::Widget::geom(); }

private:

  Fl_Boxtype _frame;
  bool _edges [4];
};

//------------------------------------------------------------------------------
// Divider Widgets

class Divider : public widget::Widget
{
public:
  
  Divider(int margin = 0);
  virtual ~Divider() = 0;
  
  void margin(int);
  int margin() const { return _margin; }
  
private:

  int _margin;
};

class Horizontal_Divider : public Divider
{
public:
  
  Horizontal_Divider(int margin = 0);
  virtual ~Horizontal_Divider() {}
  
  virtual void draw();

  virtual void dirty();
};

class Vertical_Divider : public Divider
{
public:

  Vertical_Divider(int margin = 0);
  virtual ~Vertical_Divider() {}
  
  virtual void draw();

  virtual void dirty();
};

//------------------------------------------------------------------------------
// Group Box

class Box : public widget::Widget
{
public:

  Box(const Str & label = Str());
  virtual ~Box() {}
  
  // Label.

  void label(const Str &);
  void label_widget(Widget *);
  const Str & label() const { return _label; }
  const Widget * label_widget() const { return _label_widget; }
  
  // Widget.

  virtual void draw();
  virtual void widget_del();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return widget::Widget::geom(); }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::group::Box"; }
  
private:
  
  Str _label;
  Widget * _label_widget;
  Vec2i _size_label;
};

}
}
}

#endif

