/*------------------------------------------------------------------------------
 lib/djv_ui/image_io.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/image_io.cxx"

#include "image_io.h"
#include <djv_base/sys.h>
#include <djv_ui/image_io_builtin.h>

namespace djv {
namespace ui {
namespace image_io
{

Load_Factory::Load_Factory()
{
  DJV_DBG_OBJ2(_DBG, "Load_Factory::Load_Factory");
  
  add(new Cineon_Load);
  add(new Dpx_Load);
  
  set(sys::search_path());
}

Load_Factory::~Load_Factory()
{
  DJV_DBG_OBJ2(_DBG, "Load_Factory::~Load_Factory");
  
  List<Load *> list = this->list();
  for (uint i = 0; i < list.size(); ++i)
    list[i]->widget_del();
}

Load * Load_Factory::get(djv::image::io::Load * in) const
{
  const List<plugin::Plugin *> & list = Factory::list();
  const Str name = in->name();
  for (uint i = 0; i < list.size(); ++i)
    if (list[i]->name() == name)
      return static_cast<Load *>(list[i]->copy());
  return 0;
}

List<Load *> Load_Factory::list() const
{
  const List<plugin::Plugin *> & list = Factory::list();
  List<Load *> out;
  out.resize(list.size());
  for (uint i = 0; i < list.size(); ++i)
    out[i] = dynamic_cast<Load *>(list[i]);
  return out;
}

Save_Factory::Save_Factory()
{
  DJV_DBG_OBJ2(_DBG, "Save_Factory::Save_Factory");
  
  add(new Cineon_Save);
  add(new Dpx_Save);
  add(new Lut_Save);
  add(new Ppm_Save);
  add(new Sgi_Save);
  add(new Tga_Save);
  
  set(sys::search_path());
}

Save_Factory::~Save_Factory()
{
  DJV_DBG_OBJ2(_DBG, "Save_Factory::~Save_Factory");
  
  List<Save *> list = this->list();
  for (uint i = 0; i < list.size(); ++i)
    list[i]->widget_del();
}

Save * Save_Factory::get(djv::image::io::Save * in) const
{
  const List<plugin::Plugin *> & list = Factory::list();
  const Str name = in->name();
  for (uint i = 0; i < list.size(); ++i)
    if (list[i]->name() == name)
      return static_cast<Save *>(list[i]->copy());
  return 0;
}

List<Save *> Save_Factory::list() const
{
  const List<plugin::Plugin *> & list = Factory::list();
  List<Save *> out;
  out.resize(list.size());
  for (uint i = 0; i < list.size(); ++i)
    out[i] = dynamic_cast<Save *>(list[i]);
  return out;
}

namespace
{
std::auto_ptr<Load_Factory> _load;
std::auto_ptr<Save_Factory> _save;
}

Signal<djv::image::io::Plugin *> sig;

Load_Factory * load() { return _load.get(); }
Save_Factory * save() { return _save.get(); }

void init()
{
  DJV_DBG_OBJ2(_DBG, "init");
  
  _load = std::auto_ptr<Load_Factory>(new Load_Factory);
  _save = std::auto_ptr<Save_Factory>(new Save_Factory);
}

void del()
{
  DJV_DBG_OBJ2(_DBG, "del");
  
  _load.reset();
  _save.reset();
}

}
}
}

