/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_cineon.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/image_io_cineon.cxx"

#include "image_io_builtin.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/num_int.h>
#include <djv_ui/num_float.h>

namespace djv {
namespace ui {
namespace image_io
{
using namespace djv::image::io;
typedef cineon::Load::Options Load_Options;
typedef cineon::Save::Options Save_Options;

Cineon_Load::Cineon_Load() : _plugin(0)
{
  DJV_DBG_OBJ2(_DBG, "Cineon_Load::Cineon_Load");
  
  // Widgets.
  
  group::Box * color_group = new group::Box;
  _color_widget = new choice::Widget(dpx::label_color());
  _color_widget->label(new label::Widget("Color:"));
  color_group->label_widget(_color_widget);
  ui::label::Text * color_text = new ui::label::Text("Input color space.");
  
  group::Box * print_group = new group::Box("Print");
  _black_widget = new num::Int_Slider;
  _black_widget->default_value(Load_Options().print.black);
  _black_widget->range(0, 1023);
  _black_widget->label(new label::Widget("Black:"));
  _white_widget = new num::Int_Slider;
  _white_widget->default_value(Load_Options().print.white);
  _white_widget->range(0, 1023);
  _white_widget->label(new label::Widget("White:"));
  _gamma_widget = new num::Float_Slider;
  _gamma_widget->default_value(Load_Options().print.gamma);
  _gamma_widget->range(0.01, 4.0);
  _gamma_widget->label(new label::Widget("Gamma:"));
  _soft_clip_widget = new num::Int_Slider;
  _soft_clip_widget->default_value(Load_Options().print.soft_clip);
  _soft_clip_widget->range(0, 50);
  _soft_clip_widget->label(new label::Widget("Soft Clip:"));
  
  // Layout.

  layout::group(List<layout::Item *>() << _black_widget << _white_widget <<
    _gamma_widget << _soft_clip_widget);
  
  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(color_group);
  layout::Vertical * group_layout = new layout::Vertical(color_group);
  group_layout->add(color_text);

  layout->add(print_group);  
  group_layout = new layout::Vertical(print_group);
  group_layout->add(_black_widget);
  group_layout->add(_white_widget);
  group_layout->add(_gamma_widget);
  group_layout->add(_soft_clip_widget);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _color_widget->sig.set(this, color_callback);
  _black_widget->sig.set(this, black_callback);
  _white_widget->sig.set(this, white_callback);
  _gamma_widget->sig.set(this, gamma_callback);
  _soft_clip_widget->sig.set(this, soft_clip_callback);
}

void Cineon_Load::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Cineon_Load::color_callback(int in)
{
  if (! _plugin) return;
  _options.color = dpx::COLOR(in);
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Load::black_callback(int in)
{
  if (! _plugin) return;
  _options.print.black = in;
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Load::white_callback(int in)
{
  if (! _plugin) return;
  _options.print.white = in;
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Load::gamma_callback(double in)
{
  if (! _plugin) return;
  _options.print.gamma = in;
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Load::soft_clip_callback(int in)
{
  if (! _plugin) return;
  _options.print.soft_clip = in;
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Load::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    List<Str> tmp;
    tmp = str::serialize(_options.color, dpx::label_color());
    _plugin->option(_plugin->option_list()[cineon::Save::OPTION_COLOR], &tmp);
    tmp = dpx::serialize(_options.print);
    _plugin->option(_plugin->option_list()[cineon::Save::OPTION_PRINT], &tmp);
  }
  
  callbacks(true);
}

void Cineon_Load::widget_update()
{
  callbacks(false);

  if (_plugin)
  {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[cineon::Save::OPTION_COLOR]);
    str::serialize(&tmp, &_options.color, dpx::label_color());
    tmp = _plugin->option(_plugin->option_list()[cineon::Save::OPTION_PRINT]);
    dpx::serialize(&tmp, &_options.print);
  }
  
  _color_widget->set(_options.color);
  _black_widget->set(_options.print.black);
  _white_widget->set(_options.print.white);
  _gamma_widget->set(_options.print.gamma);
  _soft_clip_widget->set(_options.print.soft_clip);

  callbacks(true);
}

Cineon_Save::Cineon_Save() : _plugin(0)
{
  // Widgets.
  
  group::Box * color_group = new group::Box;
  _color_widget = new choice::Widget(dpx::label_color());
  _color_widget->label(new label::Widget("Color:"));
  color_group->label_widget(_color_widget);
  ui::label::Text * color_text = new ui::label::Text("Output color space.");
  
  group::Box * print_group = new group::Box("Print");
  _black_widget = new num::Int_Slider;
  _black_widget->default_value(Load_Options().print.black);
  _black_widget->range(0, 1023);
  _black_widget->label(new label::Widget("Black:"));
  _white_widget = new num::Int_Slider;
  _white_widget->default_value(Load_Options().print.white);
  _white_widget->range(0, 1023);
  _white_widget->label(new label::Widget("White:"));
  _gamma_widget = new num::Float_Slider;
  _gamma_widget->default_value(Load_Options().print.gamma);
  _gamma_widget->range(0.01, 4.0);
  _gamma_widget->label(new label::Widget("Gamma:"));
  
  // Layout.
  
  layout::group(List<layout::Item *>() << _black_widget << _white_widget <<
    _gamma_widget);
  
  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(color_group);
  layout::Vertical * group_layout = new layout::Vertical(color_group);
  group_layout->add(color_text);
  
  layout->add(print_group);
  group_layout = new layout::Vertical(print_group);
  group_layout->add(_black_widget);
  group_layout->add(_white_widget);
  group_layout->add(_gamma_widget);

  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _color_widget->sig.set(this, color_callback);
  _black_widget->sig.set(this, black_callback);
  _white_widget->sig.set(this, white_callback);
  _gamma_widget->sig.set(this, gamma_callback);
}

void Cineon_Save::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Cineon_Save::color_callback(int in)
{
  if (! _plugin) return;
  _options.color = dpx::COLOR(in);
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Save::black_callback(int in)
{
  if (! _plugin) return;
  _options.print.black = in;
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Save::white_callback(int in)
{
  if (! _plugin) return;
  _options.print.white = in;
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Save::gamma_callback(double in)
{
  if (! _plugin) return;
  _options.print.gamma = in;
  plugin_update();
  sig.emit(_plugin);
}

void Cineon_Save::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    List<Str> tmp;
    tmp = str::serialize(_options.color, dpx::label_color());
    _plugin->option(_plugin->option_list()[cineon::Save::OPTION_COLOR], &tmp);
    tmp = dpx::serialize(_options.print);
    _plugin->option(_plugin->option_list()[cineon::Save::OPTION_PRINT], &tmp);
  }
  
  callbacks(true);
}

void Cineon_Save::widget_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[cineon::Save::OPTION_COLOR]);
    str::serialize(&tmp, &_options.color, dpx::label_color());
    tmp = _plugin->option(_plugin->option_list()[cineon::Save::OPTION_PRINT]);
    dpx::serialize(&tmp, &_options.print);
  }
  
  _color_widget->set(_options.color);
  _black_widget->set(_options.print.black);
  _white_widget->set(_options.print.white);
  _gamma_widget->set(_options.print.gamma);

  callbacks(true);
}

}
}
}
