/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_ppm.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/image_io_builtin.cxx"

#include "image_io_builtin.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/choice.h>

namespace djv {
namespace ui {
namespace image_io
{
using namespace djv::image::io;

Ppm_Save::Ppm_Save() : _plugin(0)
{
  // Widgets.
  
  group::Box * type_group = new group::Box;
  _type_widget = new choice::Widget(ppm::label_type());
  _type_widget->label(new label::Widget("Type:"));
  type_group->label_widget(_type_widget);
  ui::label::Text * type_text = new ui::label::Text("Pixel type.");
  
  group::Box * mode_group = new group::Box;
  _mode_widget = new choice::Widget(ppm::label_mode());
  _mode_widget->label(new label::Widget("Mode:"));
  mode_group->label_widget(_mode_widget);
  ui::label::Text * mode_text = new ui::label::Text("File mode.");
  
  // Layout.

  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(type_group);
  layout::Vertical * group_layout = new layout::Vertical(type_group);
  group_layout->add(type_text);
  
  layout->add(mode_group);
  group_layout = new layout::Vertical(mode_group);
  group_layout->add(mode_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _type_widget->sig.set(this, type_callback);
  _mode_widget->sig.set(this, mode_callback);
}

void Ppm_Save::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Ppm_Save::type_callback(int in)
{
  if (! _plugin) return;
  _options.type = ppm::TYPE(in);
  plugin_update();
  sig.emit(_plugin);
}

void Ppm_Save::mode_callback(int in)
{
  if (! _plugin) return;
  _options.mode = ppm::MODE(in);
  plugin_update();
  sig.emit(_plugin);
}

void Ppm_Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = str::serialize(_options.type, ppm::label_type());
    _plugin->option(_plugin->option_list()[ppm::Save::OPTION_TYPE], &tmp);
    tmp = str::serialize(_options.mode, ppm::label_mode());
    _plugin->option(_plugin->option_list()[ppm::Save::OPTION_MODE], &tmp);
  }
  
  callbacks(true);
}

void Ppm_Save::widget_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[ppm::Save::OPTION_TYPE]);
    str::serialize(&tmp, &_options.type, ppm::label_type());
    tmp = _plugin->option(_plugin->option_list()[ppm::Save::OPTION_MODE]);
    str::serialize(&tmp, &_options.mode, ppm::label_mode());
  }
  
  _type_widget->set(_options.type);
  _mode_widget->set(_options.mode);
  
  callbacks(true);
}

}
}
}
