/*------------------------------------------------------------------------------
 lib/djv_ui/menu_popup.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "menu.h"
#include <djv_ui/style.h>
#include <djv_ui/icon.h>
#include <FL/Fl.H>
#include <FL/Fl_Menu_Button.H>

namespace djv {
namespace ui {
namespace menu
{

class Popup::Widget : public Fl_Menu_Button
{
public:

  Widget() : Fl_Menu_Button(0, 0, 0, 0) {}

  void draw() {}
};

Popup::Popup(const List<Str> & list) : _icon(ui::icon::get("tool_dec"))
{
  frame(FL_UP_BOX);

  Frame::add(_widget = new Widget);
  
  layout::Item::size(layout::FIXED, layout::FIXED);
  
  set(list);
  
  _widget->callback(widget_callback, this);
}

void Popup::set(const List<Str> & in)
{
  menu_del();
  add(in);
  menu_end();
}

void Popup::add(const List<Str> & in)
{
  for (uint i = 0; i < in.size(); ++i)
    menu_add(in[i]);
}

void Popup::draw()
{
  Frame::draw();
  
  if (_icon)
    _icon->draw(x() + w() / 2 - _icon->w() / 2, y() + h() / 2 - _icon->h() / 2);

  if (Fl::focus() == this) draw_focus();
}
  
void Popup::dirty()
{
  Frame::dirty();

  _widget->textfont(text_font());
  _widget->textsize(style::global()->font_size());
  _widget->textcolor(text_color());
  
  size_hint(style::global()->size_tool_icon() + style::global()->margin() * 2);
}

void Popup::geom(const Box2i & in)
{
  Frame::geom(in);
  _widget->resize(in.x, in.y, in.w, in.h);
}

void Popup::widget_callback()
{
  sig.emit(_widget->value());
}

void Popup::menu(Fl_Menu_Item * in)
{
  _widget->clear();
  _widget->copy(in);
}

}
}
}
