/*------------------------------------------------------------------------------
 lib/djv_ui/messages_dialog.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "messages_dialog.h"
#include <djv_ui/prefs.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/text.h>
#include <FL/Fl_Pixmap.H>

namespace djv {
namespace ui {
namespace messages_dialog
{

Dialog::Dialog() : dialog::Base("Messages"), _show(true)
{
  // Widgets.
  
  _widget = new text::Multiline_Display;
  _show_widget = new button::Check("Show");
  button::Push * clear_widget = new button::Push("&Clear");
  button::Push * close_widget = new button::Push("Clos&e");
  
  // Layout.
  
  layout::Vertical * layout = new layout::Vertical(this);
  layout->add(_widget);
  layout->stretch(_widget);

  layout::Horizontal * h_layout = new layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(_show_widget);
  h_layout->add_stretch();
  h_layout->add(clear_widget);
  h_layout->add(close_widget);
  
  // Initialize.
  
  prefs::Prefs prefs(prefs::global(), "messages_dialog");
  List<Str> tmp;
  if (prefs.get("show", &tmp)) str::serialize(&tmp, &_show);
  
  widget_update();
  dialog_update();
  
  size(size_hint());
  close_widget->take_focus();

  // Callbacks.

  _show_widget->sig.set(this, show_callback);
  clear_widget->sig.set(this, clear_callback);
  close_widget->sig.set(this, close_callback);
}

void Dialog::set(const Str & in)
{
  _list.push_front(in);
  if (_list.size() > 100) _list.pop_back();
  widget_update();
  if (_show) show();
}

void Dialog::set(const Error & in)
{
  set(error::format(in));
}

void Dialog::widget_del()
{
  prefs::Prefs prefs(prefs::global(), "messages_dialog");
  prefs.set("show", str::serialize(_show));

  dialog::Base::widget_del();
}

void Dialog::dirty()
{
  Base::dirty();
  _size_min = max(Base::size_min(), Vec2i(350, 200));
}

void Dialog::show_callback(bool in)
{
  _show = in;
  dialog_update();
}

void Dialog::clear_callback(bool)
{
  _list.clear();
  widget_update();
}

void Dialog::close_callback(bool)
{
  hide();
}

void Dialog::widget_update()
{
  callbacks(false);
  _widget->set(str::join(_list, "\n"));
  callbacks(true);
}

void Dialog::dialog_update()
{
  callbacks(false);
  _show_widget->set(_show);
  callbacks(true);
}

namespace { std::auto_ptr<Dialog> _dialog; }

Dialog * global()
{
  if (! _dialog.get()) _dialog = std::auto_ptr<Dialog>(new Dialog);
  return _dialog.get();
}


void message(const Str & in) { global()->set(in); }
void error(const Error & in) { global()->set(in); }

void del()
{
  if (_dialog.get()) _dialog->widget_del();
  _dialog.release();
}

}
}
}

