/*------------------------------------------------------------------------------
 lib/djv_ui/scroll.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/scroll.cxx"

#include "scroll.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <FL/Fl.H>
#include <FL/Fl_Box.H>

namespace djv {
namespace ui {
namespace scroll
{

Widget::Widget() : Fl_Scroll(0, 0, 0, 0), _resizable(true)
{
  Fl_Scroll::end();
  Fl_Scroll::box(FL_THIN_DOWN_FRAME);
  //Fl_Scroll::type(Fl_Scroll::BOTH);
  //Fl_Scroll::scrollbar.slider(FL_THIN_UP_BOX);
  //Fl_Scroll::hscrollbar.slider(FL_THIN_UP_BOX);
  
  add(_dummy_widget = new Fl_Box(0, 0, 0, 0));
  _dummy_widget->box(FL_NO_BOX);
  _dummy_widget->hide();
  
  layout::Vertical * layout = new layout::Vertical(this);
  //layout->margin(0);
}

int Widget::handle(int in)
{
  switch (widget::Base::handle(in))
  {
    case -1: return 0;
    case 1: return 1;
  }
  return Fl_Scroll::handle(in);
}

void Widget::draw()
{
  Fl_Scroll::draw();
  widget::Base::draw();
}

void Widget::resizable(bool in)
{
  _resizable = in;
}

void Widget::dirty()
{
  DJV_DBG_OBJ2(_DBG, "Widget::dirty");
  
  widget::Base::dirty();

  size_hint(Vec2i(Fl_Scroll::scrollbar.w(), Fl_Scroll::hscrollbar.h()) +
    1 * 2);

  _size_min = style::global()->size_text();
}

void Widget::geom(const Box2i & in)
{
  DJV_DBG_OBJ2(_DBG, "Widget::geom");
  DJV_DBG_PRN("in = " << in);
  
  Fl_Scroll::resize(in.x, in.y, in.w, in.h);
  Item::geom(in);
  
  if (_resizable)
  {
    const Vec2i & size = layout()->size_hint();
    DJV_DBG_PRN("size = " << size);
  
    Box2i frame(in.x + 1, in.y + 1, in.w - 1 * 2, in.h - 1 * 2);

    // XXX Determine scrollbar visibility. FLTK doesn't know because draw()
    // hasn't been called yet.

    bool scrollbar_v = false, scrollbar_h = false;
    if (size.y > frame.h)
    {
      scrollbar_v = true;
      frame.w -= Fl_Scroll::scrollbar.w();
    }
    if (size.x > frame.w)
    {
      scrollbar_h = true;
      frame.h -= Fl_Scroll::hscrollbar.h();
      if (! scrollbar_v && size.y > frame.h) {
        scrollbar_v = true;
        frame.w -= Fl_Scroll::scrollbar.w();
      }
    }
    //frame.w = math::max(frame.w, size.x);
    //frame.h = math::max(frame.h, size.y);
    DJV_DBG_PRN("frame = " << frame.size);
    DJV_DBG_PRN("scrollbar = " << scrollbar_h << " " << scrollbar_v);

    // XXX

    _dummy_widget->resize(frame.x, frame.y, frame.w, frame.h);
    layout()->geom(frame);
  }
}

}
}
}
