/*------------------------------------------------------------------------------
 lib/djv_ui/text_widget.h
 
 Text Widgets
 ------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_TEXT_WIDGET_H
#define DJV_UI_TEXT_WIDGET_H

#include <djv_ui/group.h>

namespace djv {
namespace ui {
namespace text
{

//------------------------------------------------------------------------------
// Text Edit

class Edit : public group::Frame
{
public:

  Edit(const Str & = Str());
  virtual ~Edit() {}
  
  // Value.
  
  void set(const Str &);
  const Str & get() const { return _value; }
  Signal<const Str &> sig, action_sig, change_sig;
  
  // Widget.

  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const { return Frame::text_font(); }
  virtual const Fl_Color & text_color() const { return Frame::text_color(); }
  
  virtual int handle(int);
  
  // Layout.

  virtual void size_str(const Str &);
  virtual const Str & size_str() const { return _size_str; }
  
  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return Frame::geom(); }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::text::Widget"; }

private:

  class Widget;

  DJV_FL_WIDGET_CALLBACK(Edit, widget_callback);

  void widget_update();

  Str _value;
  Str _size_str;
  Widget * _widget;
};

//------------------------------------------------------------------------------
// Text Display

class Display : public group::Frame
{
public:

  Display(const Str & = Str());
  virtual ~Display() {}

  // Value.

  void set(const Str &);
  const Str & get() const { return _value; }
  
  // Widget.
  
  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const { return Frame::text_font(); }
  virtual const Fl_Color & text_color() const { return Frame::text_color(); }
  
  // Layout.

  virtual void size_str(const Str &);
  virtual const Str & size_str() const { return _size_str; }

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return Frame::geom(); }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::text::Display"; }

private:

  class Widget;

  void widget_update();

  Str _value;
  Str _size_str;
  Widget * _widget;
};

//------------------------------------------------------------------------------
// Multiline Text Display

class Multiline_Display : public group::Frame
{
public:

  Multiline_Display(const Str & = Str());
  virtual ~Multiline_Display() {}
  
  // Value.

  void set(const Str &);
  const Str & get() const { return _value; }
  
  // Widget.
  
  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const { return Frame::text_font(); }
  virtual const Fl_Color & text_color() const { return Frame::text_color(); }
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return Frame::geom(); }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::text::Multiline_Display"; }

private:

  class Widget;

  void widget_update();

  Str _value;
  Widget * _widget;
};

}
}
}

#endif

