/*------------------------------------------------------------------------------
 plugin/djv_jpeg/plugin.h

 Joint Photographic Experts Group (JPEG) Image I/O
 -------------------------------------------------
 
 Requires:
 * libjpeg - http://www.ijg.org
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_JPEG_PLUGIN_H
#define DJV_JPEG_PLUGIN_H

#include <djv_image/io.h>
namespace libjpeg // XXX WIN32 conflicts?
{
extern "C"
{
#include <jpeglib.h>
}
}
#include <stdio.h>
#include <setjmp.h>

using namespace djv;

//------------------------------------------------------------------------------
// Plugin Information

static const Str name = "JPEG";

static const List<Str> extension_list = List<Str>() << ".jpeg" << ".jpg" <<
  ".jfif";

//------------------------------------------------------------------------------
// Errors

struct Jpeg_Error
{
  struct libjpeg::jpeg_error_mgr pub;
  char msg [JMSG_LENGTH_MAX];
  jmp_buf jump;
};

extern "C"
{
void djv_jpeg_error(libjpeg::j_common_ptr);
void djv_jpeg_warn(libjpeg::j_common_ptr, int);
}

//------------------------------------------------------------------------------
// Load Plugin

class Load : public image::io::Load
{
public:

  Load();
  ~Load();
  
  plugin::Plugin * copy() const { return new Load; }

  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, image::Info *, image::Tag *, image::Time *)
    throw (Error);
  void open(const File & in, List<image::Info> * info, image::Tag * tag,
    image::Time * time) throw (Error)
  { image::io::Load::open(in, info, tag, time); }
  const image::Image * get(int frame, int layer, int proxy) throw (Error);

private:

  void _open(const File &, image::Info *, image::Tag *) throw (Error);
  void _close();

  File _file;
  FILE * _f;
  libjpeg::jpeg_decompress_struct _jpeg;
  bool _jpeg_init;
  Jpeg_Error _jpeg_error;
  image::Image _image;
  image::Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public image::io::Save
{
public:

  Save();
  ~Save();
  
  plugin::Plugin * copy() const;
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  struct Options
  {
    Options() : quality(100) {}
    int quality;
  };
  enum OPTION { OPTION_QUALITY };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const image::Info &, const image::Tag &,
    const image::Time &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:
  
  void _open(const File &, const image::Tag &) throw (Error);
  void _close();

  Options _options;
  File _file;
  FILE * _f;
  libjpeg::jpeg_compress_struct _jpeg;
  bool _jpeg_init;
  Jpeg_Error _jpeg_error;
  image::Info _info;
  image::Image _image;
};

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_save_option();

#endif

