/*------------------------------------------------------------------------------
 plugin/djv_png/plugin.h

 Portable Network Graphics (PNG) Image I/O
 -----------------------------------------

 Requires:
 * libpng - http://www.libpng.org

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_PNG_PLUGIN_H
#define DJV_PNG_PLUGIN_H

#include <djv_image/io.h>
#include <png.h>

using namespace djv;

//------------------------------------------------------------------------------
// Plugin Information

static const Str name = "PNG";

static const List<Str> extension_list = List<Str>() << ".png";

//------------------------------------------------------------------------------
// Errors

struct Png_Error
{
  char msg [str::cstr_size];
};

extern "C"
{
void djv_png_error(png_structp, png_const_charp);
void djv_png_warning(png_structp, png_const_charp);
}

//------------------------------------------------------------------------------
// Load Plugin

class Load : public image::io::Load
{
public:

  Load();
  ~Load();

  plugin::Plugin * copy() const { return new Load; }
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, image::Info *, image::Tag *, image::Time *)
    throw (Error);
  void open(const File & in, List<image::Info> * info, image::Tag * tag,
    image::Time * time) throw (Error)
  { image::io::Load::open(in, info, tag, time); }
  const image::Image * get(int frame, int layer, int proxy) throw (Error);

private:

  void _open(const File &, image::Info *) throw (Error);
  void _close();

  File _file;
  FILE * _f;
  png_structp _png;
  png_infop _png_info, _png_info_end;
  Png_Error _png_error;
  image::Image _image;
  image::Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public image::io::Save
{
public:

  Save();
  ~Save();
  
  plugin::Plugin * copy() const { return new Save; }
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, const image::Info &, const image::Tag &,
    const image::Time &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:

  void _open(const File &, const image::Tag &) throw (Error);
  void _close();
  
  File _file;
  FILE * _f;
  png_structp _png;
  png_infop _png_info;
  Png_Error _png_error;
  image::Info _info;
  image::Image _image;
};

#endif

