/*------------------------------------------------------------------------------
 plugin/djv_quicktime_apple/plugin.h
 
 Apple QuickTime Image I/O
 -------------------------
 
 Requires: 
 * QuickTime - http://www.apple.com/quicktime

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_QUICKTIME_PLUGIN_H
#define DJV_QUICKTIME_PLUGIN_H

#include <djv_image/io.h>
//#include <QuickTime.h>
#include <Movies.h>

using namespace djv;

//------------------------------------------------------------------------------
// Plugin Information

static const Str name = "QuickTime";

static const List<Str> extension_list = List<Str>() << ".qt" << ".mov"
  << ".avi";

//------------------------------------------------------------------------------
// Plugin Options

enum CODEC { CODEC_RAW, CODEC_JPEG, CODEC_MJPEGA, CODEC_MJPEGB, CODEC_H263,
  CODEC_H264, CODEC_DVCNTSC, CODEC_DVCPAL };

enum QUALITY { QUALITY_LOSSLESS, QUALITY_MIN, QUALITY_MAX, QUALITY_LOW,
  QUALITY_NORMAL, QUALITY_HIGH };

//------------------------------------------------------------------------------
// Load Plugin

class Load : public image::io::Load
{
public:

  Load() throw (Error);
  ~Load();

  plugin::Plugin * copy() const { return new Load; }
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, image::Info *, image::Tag *, image::Time *)
    throw (Error);
  void open(const File & in, List<image::Info> * info, image::Tag * tag,
    image::Time * time) throw (Error)
  { image::io::Load::open(in, info, tag, time); }
  const image::Image * get(int frame, int layer, int proxy) throw (Error);

private:

  void _close();
  
  File _file;
  image::Info _info;
  image::Time _time;
  Movie _movie;
  GWorldPtr _gworld;
  image::Image _image, _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public image::io::Save
{
public:

  Save() throw (Error);
  ~Save();
  
  plugin::Plugin * copy() const;
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  struct Options
  {
    Options() : codec(CODEC_JPEG), quality(QUALITY_NORMAL) {}
    CODEC codec;
    QUALITY quality;
  };
  enum OPTION { OPTION_CODEC, OPTION_QUALITY };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const image::Info &, const image::Tag &,
    const image::Time &) throw (Error);
  void close() throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:

  void _open(const File &, const image::Info &) throw (Error);
  
  Options _options;
  File _file;
  int _frame_duration;
  short _f;
  Movie _movie;
  Track _track;
  Media _media;
  GWorldPtr _gworld;
  image::Info _info;
  image::Image _image;
};

//------------------------------------------------------------------------------
// Functions

void quicktime_init() throw (Error);
void quicktime_del();

CodecType quicktime_codec(CODEC);
CodecQ quicktime_quality(QUALITY);

//------------------------------------------------------------------------------
// Labels

const Str
  error_init = "Initialize = %1",
  error_version = "Version = %1",
  error_open = "Open = %1",
  error_gworld = "GWorld = %1",
  error_track = "Track = %1",
  error_media = "Media = %1",
  error_media_begin = "Media Begin = %1",
  error_media_end = "Media End = %1",
  error_media_add = "Media Add = %1",
  error_media_sample = "Media Sample = %1",
  error_movie_add = "Movie Add = %1",
  error_compress = "Compress = %1",
  error_handle = "Handle = %1";

const List<Str> & label_codec();
const List<Str> & label_quality();
const List<Str> & label_save_option();

#endif

