/*------------------------------------------------------------------------------
 plugin/djv_vlut/load.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_vlut/load.cxx"

#include "plugin.h"
#include <FL/x.h>

extern "C"
{
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_load() { return new Load; }
}

void Load::open(const File &, image::Info * info, image::Tag *,
  image::Time *) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::open");
  DJV_DBG_PRN("in = " << in);
  
  // Information.

  int size = 0;

#if defined(DJV_VLUT_XORG)

  if (! XF86VidModeGetGammaRampSize(fl_display, fl_screen, &size))
    throw Error(name(), image::io::error_open);
  
#elif defined(DJV_VLUT_XSGI)

  int maps = 0, precision = 0, attr = 0;
  if (! XSGIvcQueryScreenGammaMaps(fl_display, fl_screen, &maps))
    throw Error(name(), image::io::error_open);
  if (maps <= 0)
    throw Error(name(), image::io::error_read);
  if (! XSGIvcQueryGammaMap(fl_display, fl_screen, 0, &size, &precision,
    &attr)
  )
    throw Error(name(), image::io::error_open);
  if (attr & XSGIVC_GMReadLock)
    throw Error(name(), image::io::error_open);

#elif defined(DJV_VLUT_WIN32)

  size = 256;

#endif

  _info.size = Vec2i(size, 1);
  _info.pixel = image::pixel::RGB_U16;
  *info = _info;

  _image.set(_info);
  _tmp.set(_info);
}

const image::Image * Load::get(int, int, int) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::get");
  
  // Read.

  const int size = _info.size.x;
  image::pixel::U16_T * p =
    reinterpret_cast<image::pixel::U16_T *>(_tmp.data());

#if defined(DJV_VLUT_XORG)
  
  XF86VidModeGetGammaRamp(fl_display, fl_screen, size, p, p + size,
    p + size * 2);
  
#elif defined(DJV_VLUT_XSGI)

  uint16_t * tmp [] = { 0, 0, 0 };
  int _size = 0;
  XSGIvcQueryGammaColors(fl_display, fl_screen, 0, XSGIVC_COMPONENT_RED,
    &_size, tmp);
  XSGIvcQueryGammaColors(fl_display, fl_screen, 0, XSGIVC_COMPONENT_GREEN,
    &_size, tmp + 1);
  XSGIvcQueryGammaColors(fl_display, fl_screen, 0, XSGIVC_COMPONENT_BLUE,
    &_size, tmp + 2);
  mem::copy(tmp[0], p, size * 2);
  mem::copy(tmp[1], p + size, size * 2);
  mem::copy(tmp[2], p + size * 2, size * 2);
  if (tmp[0]) XFree(tmp[0]);
  if (tmp[1]) XFree(tmp[1]);
  if (tmp[2]) XFree(tmp[2]);

#elif defined(DJV_VLUT_WIN32)

  ::GetDeviceGammaRamp(handle()->display, p);

#endif
  
#if defined(DJV_DBG)
  for (int i = 0; i < size; ++i) DJV_DBG_PRN(i << " = " << p[i]);
#endif
  
  image::data::planar_interleave(_tmp, &_image);
  
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

