/*------------------------------------------------------------------------------
 plugin/djv_vlut/plugin.h
 
 Video Lookup Table (VLUT) Image I/O
 -----------------------------------
 
 Requires: 
 * X11 Video Mode Extension - http:://www.x.org
 * X11 Video Control Extension - http:://www.sgi.org/irix

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VLUT_PLUGIN_H
#define DJV_VLUT_PLUGIN_H

#include <djv_image/io.h>
#if defined(DJV_VLUT_XORG)
#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>
#elif defined(DJV_VLUT_XSGI)
#include <X11/Xlib.h>
#include <X11/extensions/XSGIvc.h>
#elif defined(DJV_VLUT_WIN32)
#include <windows.h>
//#include <wingdi.h>
#endif

using namespace djv;

//------------------------------------------------------------------------------
// Plugin Information

static const Str name = "VLUT";

static const List<Str> extension_list = List<Str>() << ".vlut";

//------------------------------------------------------------------------------
// Video LUT Handle

#if defined(DJV_VLUT_WIN32)

class Handle
{
public:

  Handle() throw (Error);
  ~Handle();

  HDC display;
};

#endif

//------------------------------------------------------------------------------
// Load Plugin

class Load : public image::io::Load
{
public:
  
  plugin::Plugin * copy() const { return new Load; }
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, image::Info *, image::Tag *, image::Time *)
    throw (Error);
  void open(const File & in, List<image::Info> * info, image::Tag * tag,
    image::Time * time) throw (Error)
  { image::io::Load::open(in, info, tag, time); }
  const image::Image * get(int frame, int layer, int proxy) throw (Error);

private:

  image::Info _info;
  image::Image _image;
  image::Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public image::io::Save
{
public:
  
  plugin::Plugin * copy() const { return new Save; }
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, const image::Info &, const image::Tag &,
    const image::Time &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:

  image::Info _info;
  image::Image _image, _tmp;
};

//------------------------------------------------------------------------------
// Global

#if defined(DJV_VLUT_WIN32)
Handle * handle();
#endif

#endif

